import { Injectable } from '@angular/core';
import {MutableKeyStore, MutablePgpKeyStore, UnsafeKeyStore} from '@app/_helpers';
import { hobaParseChallengeHeader } from '@src/assets/js/hoba.js';
import { signChallenge } from '@src/assets/js/hoba-pgp.js';
import {environment} from '@src/environments/environment';
import {HttpClient} from '@angular/common/http';

const origin = 'http://localhost:4444';
const pgpKeyStore = new UnsafeKeyStore();

@Injectable({
  providedIn: 'root'
})
export class AuthService {
  sessionToken: any;
  sessionLoginCount = 0;
  privateKey: any;
  mutableKeyStore: MutableKeyStore = new MutablePgpKeyStore();

  constructor(
    private http: HttpClient
  ) {
    if (sessionStorage.getItem(btoa('CICADA_SESSION_TOKEN'))) {
      this.sessionToken = sessionStorage.getItem(btoa('CICADA_SESSION_TOKEN'));
    }
    if (localStorage.getItem(btoa('CICADA_PRIVATE_KEY'))) {
      this.privateKey = localStorage.getItem(btoa('CICADA_PRIVATE_KEY'));
    }
  }

  setState(s): void {
    (document.getElementById('state') as HTMLInputElement).value = s;
  }

  getWithToken(): void {
    const xhr = new XMLHttpRequest();
    xhr.responseType = 'text';
    xhr.open('GET', origin + window.location.search.substring(1));
    xhr.setRequestHeader('Authorization', 'Bearer ' + this.sessionToken);
    xhr.setRequestHeader('Content-Type', 'application/json');
    xhr.setRequestHeader('x-cic-automerge', 'none');
    xhr.addEventListener('load', (e) => {
      if (xhr.status === 401) {
        throw new Error('login rejected');
      }
      this.sessionLoginCount++;
      this.setState('click to perform login ' + this.sessionLoginCount + ' with token ' + this.sessionToken);
      console.log('received', xhr.responseText);
      return;
    });
    xhr.send();
  }

  sendResponse(hobaResponseEncoded): void {
    const xhr = new XMLHttpRequest();
    // xhr.responseType = 'arraybuffer';
    xhr.responseType = 'text';
    xhr.open('GET', origin + window.location.search.substring(1));
    xhr.setRequestHeader('Authorization', 'HOBA ' + hobaResponseEncoded);
    xhr.setRequestHeader('Content-Type', 'application/json');
    xhr.setRequestHeader('x-cic-automerge', 'none');
    xhr.addEventListener('load', (e) => {
      if (xhr.status === 401) {
        throw new Error('login rejected');
      }
      this.sessionToken = xhr.getResponseHeader('Token');
      sessionStorage.setItem(btoa('CICADA_SESSION_TOKEN'), this.sessionToken);
      this.sessionLoginCount++;
      this.setState('click to perform login ' + this.sessionLoginCount + ' with token ' + this.sessionToken);
      console.log('received', xhr.responseText);
      return;
    });
    xhr.send();
  }

  getChallenge(): void {
    const xhr = new XMLHttpRequest();
    xhr.responseType = 'arraybuffer';
    xhr.open('GET', origin + window.location.search.substring(1));
    xhr.onload = (e) => {
      if (xhr.status === 401) {
        const authHeader = xhr.getResponseHeader('WWW-Authenticate');
        const o = hobaParseChallengeHeader(authHeader);
        this.loginResponse(o).then();
      }
    };
    xhr.send();
  }


  login(): boolean {
    if (this.sessionToken !== undefined) {
      try {
        this.getWithToken();
        return true;
      } catch (e) {
        console.error('login token failed', e);
      }
    } else {
      try {
        const o = this.getChallenge();
        return true;
      } catch (e) {
        console.error('login challenge failed', e);
      }
    }
    return false;
  }


  async loginResponse(o): Promise<any> {
    const r = await signChallenge(o.challenge, o.realm, origin, pgpKeyStore);
    this.sendResponse(r);
  }

  loginView(): void {
    document.getElementById('one').style.display = 'none';
    document.getElementById('two').style.display = 'block';
    this.setState('click to log in with PGP key ' + pgpKeyStore.keyId().toHex());
  }

  async setKey(privateKeyArmored): Promise<boolean> {
    console.log('settings pk' + privateKeyArmored);
    try {
      await pgpKeyStore.set(privateKeyArmored);
      localStorage.setItem(btoa('CICADA_PRIVATE_KEY'), privateKeyArmored);
      await this.mutableKeyStore.importPrivateKey(privateKeyArmored);
    } catch (e) {
      console.error('failed setting key', e);
      return false;
    }
    this.loginView();
    return true;
  }

  logout(): void {
    sessionStorage.removeItem(btoa('CICADA_SESSION_TOKEN'));
    window.location.reload(true);
  }

  async getPublicKeys(): Promise<void> {
    this.http.get(`${environment.publicKeysUrl}/keys.asc`).subscribe(async res => {
      const armoredPublicKeys = res;
      await this.mutableKeyStore.loadKeyring();
      await this.mutableKeyStore.importPublicKey(armoredPublicKeys);
    }, error => {
      console.error('There was an error!', error);
    });
    if (this.privateKey !== undefined) {
      await this.mutableKeyStore.importPrivateKey(this.privateKey);
    }
  }
}
